﻿/* Created by SharpDevelop.
 * User: johann
 * Date: 02/03/2012
 * Time: 17:56
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using SpringCardPCSC;
using SpringCardNFC;

namespace SpringCardApplication
{
	public partial class MainForm : Form
	{
		enum Status
		{
			Idle,
			RecvOnly_Init,
			RecvOnly_Ready,
			SendOnly_Init,
			SendOnly_Ready,
			RecvThenSend_Init,
			RecvThenSend_RecvReady,
			RecvThenSend_RecvDone,
			RecvThenSend_SendReady,
			SendThenRecv_Init,
			SendThenRecv_SendReady,
			SendThenRecv_SendDone,
			SendThenRecv_RecvReady
		}		
		
		ShadowForm shade = null;
		NdefDisplayForm display = null;
		SCardReader reader = null;
		RtdControl control = null;
		
		LLCP llcp = null;
		
		Status status = Status.Idle;
		
		bool ready = false;
		
		public MainForm()
		{
			InitializeComponent();
			
			Settings s = new Settings();
			if (s.ShowConsole)
			{
				SpringCardUtils.SystemConsole.Show();
				Trace.Listeners.Add(new ConsoleTraceListener());
			}
			miShowConsole.Checked = s.ShowConsole;
			
			ready = true;
		}
		
		void PerformStatusChange()
		{
			switch (status)
			{
				case Status.RecvOnly_Init :
					AnnounceReceiver(true);
					toolStripStatusLabelMode.Text = "Receiver";					
					status = Status.RecvOnly_Ready;
					break;
				case Status.RecvOnly_Ready :
					break;
					
				case Status.SendOnly_Init :
					toolStripStatusLabelMode.Text = "Sender";
					AnnounceReceiver(false);
					status = Status.SendOnly_Ready;
					break;
				case Status.SendOnly_Ready :
					break;
					
				case Status.RecvThenSend_Init :
					toolStripStatusLabelMode.Text = "Receiver (will send afterwards)";
					AnnounceReceiver(true);
					status = Status.RecvThenSend_RecvReady;
					break;
				case Status.RecvThenSend_RecvReady :
					break;					
				case Status.RecvThenSend_RecvDone :
					toolStripStatusLabelMode.Text = "Sender (receive OK)";
					AnnounceReceiver(false);
					status = Status.RecvThenSend_SendReady;
					break;
				case Status.RecvThenSend_SendReady :
					break;
					
				case Status.SendThenRecv_Init :
					toolStripStatusLabelMode.Text = "Sender (will receive afterwards)";
					AnnounceReceiver(false);
					status = Status.SendThenRecv_SendReady;
					break;
				case Status.SendThenRecv_SendReady :
					break;
				case Status.SendThenRecv_SendDone :
					toolStripStatusLabelMode.Text = "Receiver (sent OK)";
					AnnounceReceiver(true);
					status = Status.SendThenRecv_RecvReady;
					break;
				case Status.SendThenRecv_RecvReady :
					break;
					
				default :
					toolStripStatusLabelMode.Text = status.ToString();
					break;
			}
		}

		delegate void ReaderStatusChangedInvoker(uint ReaderState, CardBuffer CardAtr);
		void ReaderStatusChanged(uint ReaderState, CardBuffer CardAtr)
		{
			/* The ReaderStatusChanged function is called as a delegate (callback) by the SCardReader object    */
			/* within its backgroung thread. Therefore we must use the BeginInvoke syntax to switch back from   */
			/* the context of the background thread to the context of the application's main thread. Overwise   */
			/* we'll get a security violation when trying to access the window's visual components (that belong */
			/* to the application's main thread and can't be safely manipulated by background threads).         */
			if (InvokeRequired)
			{
				this.BeginInvoke(new ReaderStatusChangedInvoker(ReaderStatusChanged), ReaderState, CardAtr);
				return;
			}
			
			string s = SCARD.ReaderStatusToString(ReaderState);
			
			Trace.WriteLine("ReaderStatusChanged: " + s);
			eReaderStatus.Text = s;
			
			if (CardAtr != null)
			{
				s = CardAtr.AsString(" ");
				eCardAtr.Text = s;
				Trace.WriteLine("\t" + s);
			} else
			{
				eCardAtr.Text = "";
			}
			
			if (ReaderState == SCARD.STATE_UNAWARE)
			{
				if (llcp != null)
				{
					llcp.Stop();
					llcp = null;
				}
				
				MessageBox.Show("The reader we were working on has disappeared from the system. This application will terminate now.",
				                "The reader has been removed",
				                MessageBoxButtons.OK,
				                MessageBoxIcon.Exclamation);
				
				Application.Exit();
				
			}
			else if ((ReaderState & SCARD.STATE_UNAVAILABLE) != 0)
			{
				Trace.WriteLine("Card unavailable");
			}
			else if ((ReaderState & SCARD.STATE_MUTE) != 0)
			{
				Trace.WriteLine("Card mute");
			}
			else if ((ReaderState & SCARD.STATE_INUSE) != 0)
			{
				Trace.WriteLine("Card in use");
			}			
			else if ((ReaderState & SCARD.STATE_EMPTY) != 0)
			{
				Trace.WriteLine("Card absent");				

				if (llcp != null)
				{
					Trace.WriteLine("*** Target lost ***");
					
					llcp.Stop();
					llcp = null;
					
					PerformStatusChange();
				}
				
				pControl.Enabled = true;
			}
			else if ((ReaderState & SCARD.STATE_PRESENT) != 0)
			{
				Trace.WriteLine("Card present");
							
				if (llcp == null)
				{
					/* Try to work with this card, it may be a LLCP peer */
					pControl.Enabled = false;
					Trace.WriteLine("*** Starting LLCP ***");
					
					llcp = new LlcpInitiator(reader);
					
					switch (status)
					{
						case Status.RecvOnly_Ready :
						case Status.RecvThenSend_RecvReady :
						case Status.SendThenRecv_RecvReady :
							/* We are a receiver */
							StartReceiver();
							break;
						
						case Status.SendOnly_Ready :
						case Status.RecvThenSend_SendReady :
						case Status.SendThenRecv_SendReady :
							/* We are a sender */
							StartSender();
							break;
						
						default :
							break;
					}
					
					llcp.Start();
				}
			}
		}
		
		void StartReceiver()
		{
			/* Instanciate a SNEP server - if a message arrives, it will call our OnNdefMessageReceived method */
			Trace.WriteLine("Create (and start) SNEP Server (receiver)");
			llcp.RegisterServer(new SNEP_Server(OnNdefMessageReceived));
		}
		
		void StartSender()
		{
			/* Instanciate a SNEP client */
			Ndef myNdef = CreateMyNdefMessage();

			if (myNdef != null)
			{
				Trace.WriteLine("Create SNEP Client (sender)");
				SNEP_Client client = new SNEP_Client(myNdef);
				client.OnMessageSent = new SNEP_Client.SNEPMessageSentCallback(OnNdefMessageSent);
				client.MessageSentCallbackOnAcknowledge = true;
				llcp.RegisterClient(client);
			} else
			{
				Trace.WriteLine("Can't create SNEP Client: nothing to send");
			}
		}

		void MainFormShown(object sender, EventArgs e)
		{
			if (true)
			{
				SplashForm splash = new SplashForm();
				splash.ShowDialog();
			}
			
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			
			if (readerSelect.SelectedReader == null)
			{
				readerSelect.Preselect("SpringCard|NFC|tactless");
				for (;;)
				{
					readerSelect.ShowDialog();
					if (readerSelect.SelectedReader != null)
						break;
					
					if (MessageBox.Show("This application can't run without a reader. Do you want to leave the application now ?",
					                    "No reader selected",
					                    MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
					{
						Application.Exit();
						return;
					}
				}
			}
			
			eReaderName.Text = readerSelect.SelectedReader;
			eReaderStatus.Text = "";
			eCardAtr.Text = "";
			
			shade.Dispose();
			shade = null;
			
			reader = new SCardReader(eReaderName.Text);
			
			Settings s = new Settings();
			
			switch (s.SelectedMode)
			{
				case Settings.MODE_RECV_ONLY :
					rbRecvOnly.Checked = true;
					break;
				case Settings.MODE_SEND_ONLY :
					rbSendOnly.Checked = true;
					break;
				case Settings.MODE_RECV_THEN_SEND :
					rbRecvThenSend.Checked = true;
					break;
				case Settings.MODE_SEND_THEN_RECV :
					rbSendThenRecv.Checked = true;
					break;
				case Settings.MODE_SEND_AND_RECV :
				default :
					rbSendThenRecv.Checked = true;
					break;
			}
			
			CbModeCheckedChanged(null, null);
			
			if ((s.SelectedType >= 0) && (s.SelectedType < cbNdefType.Items.Count))
				cbNdefType.SelectedIndex = s.SelectedType;
			else
				cbNdefType.SelectedIndex = 0;
			CbNdefTypeSelectedIndexChanged(null, null);
			
			reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChanged));
		}
		
		void MainFormFormClosed(object sender, FormClosedEventArgs e)
		{
			if (llcp != null)
			{
				llcp.Stop();
				llcp = null;
			}

			if (reader != null)
			{
				reader.StopMonitor();
				reader = null;
			}
			
			Settings s = new Settings();
			
			if (rbRecvOnly.Checked) s.SelectedMode = Settings.MODE_RECV_ONLY; else
			if (rbSendOnly.Checked) s.SelectedMode = Settings.MODE_SEND_ONLY; else
			if (rbRecvThenSend.Checked) s.SelectedMode = Settings.MODE_RECV_THEN_SEND; else
			if (rbSendThenRecv.Checked) s.SelectedMode = Settings.MODE_SEND_THEN_RECV; else s.SelectedMode = Settings.MODE_SEND_AND_RECV;

			s.SelectedType = cbNdefType.SelectedIndex;

			s.Save();
		}
		
		void MainFormLoad(object sender, EventArgs e)
		{
			shade = new ShadowForm();
			shade.ShowShadow(this);
		}
		
		void AboutToolStripMenuItemClick(object sender, EventArgs e)
		{
			shade = new ShadowForm();
			shade.ShowShadow(this);
			AboutForm f = new AboutForm();
			f.ShowDialog();
			shade.Dispose();
			shade = null;
		}
		
		
		void ChangeReader()
		{
			if (reader != null)
			{
				reader.StopMonitor();
				reader = null;
			}
			
			eReaderStatus.Text = "";
			eCardAtr.Text = "";

			shade = new ShadowForm();
			shade.ShowShadow(this);
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			readerSelect.SelectedReader = eReaderName.Text;
			readerSelect.ShowDialog();
			if (readerSelect.SelectedReader != null)
				eReaderName.Text = readerSelect.SelectedReader;
			shade.Dispose();
			shade = null;
			
			reader = new SCardReader(eReaderName.Text);

			reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChanged));
		}
		
		void MainFormFormClosing(object sender, FormClosingEventArgs e)
		{

		}
		
		void imgLogoClick(object sender, EventArgs e)
		{
			System.Diagnostics.Process.Start("http://www.springcard.com");
		}
		
		void BtnReaderChangeClick(object sender, System.EventArgs e)
		{
			ChangeReader();
		}
		
		void QuitToolStripMenuItemClick(object sender, EventArgs e)
		{
			Close();
		}
		
		Ndef CreateMyNdefMessage()
		{
			if (control == null)
				return null;
			
			return control.GetContent();
		}
		

		delegate void OnNdefMessageReceivedInvoker(Ndef hisNdef);
		void OnNdefMessageReceived(Ndef hisNdef)
		{
			/* The OnNdefMessageReceived function is called as a delegate (callback) within the NfcLlcp         */
			/* object's backgroung thread. Therefore we must use the BeginInvoke syntax to switch back from     */
			/* the context of the background thread to the context of the application's main thread. Overwise   */
			/* we'll get a security violation when trying to access the window's visual components (that belong */
			/* to the application's main thread and can't be safely manipulated by background threads).         */
			if (InvokeRequired)
			{
				this.BeginInvoke(new OnNdefMessageReceivedInvoker(OnNdefMessageReceived), hisNdef);
				return;
			}
			
			/* Message received, what shall we do now ? */
			/* ---------------------------------------- */
			
			Trace.WriteLine("*** NDEF message received ***");
			
			switch (status)
			{
				case Status.SendThenRecv_RecvReady :
					/* Thank you very much, back to send mode */
					status = Status.SendThenRecv_Init;
					if (llcp != null)
						llcp.StopAndResetPeer();
					break;
				
				case Status.RecvThenSend_RecvReady :
					/* Thank you very much, going to send */
					status = Status.RecvThenSend_RecvDone;
					if (llcp != null)
						llcp.StopAndResetPeer();					
					break;
					
				default :
					break;
			}	
			
			if (display != null)
			{
				/* We already are displaying a message, we must wait until the user closes the form */
				return;
			}
			
			display = new NdefDisplayForm();
			
			//  	  string t = (new CardBuffer(hisNdef.GetBytes())).AsString(" ");
			//  	  MessageBox.Show(t);

			shade = new ShadowForm();
			shade.ShowShadow(this);

			display.SetNdef(hisNdef);
			display.ShowDialog();
			display = null;
			
			shade.Dispose();
			shade = null;
		}
		
		delegate void OnNdefMessageSentInvoker();
		void OnNdefMessageSent()
		{
			if (InvokeRequired)
			{
				this.BeginInvoke(new OnNdefMessageSentInvoker(OnNdefMessageSent));
				return;
			}
			
			Trace.WriteLine("*** NDEF message sent ***");
			
			switch (status)
			{
				case Status.SendThenRecv_SendReady :
					/* Sent OK */
					status = Status.SendThenRecv_SendDone;
					break;
				
				case Status.RecvThenSend_SendReady :
					/* Send OK */
					status = Status.RecvThenSend_Init;
					break;
					
				default :
					break;
			}	
		}
		
		void CbNdefTypeSelectedIndexChanged(object sender, EventArgs e)
		{
			if (control != null)
			{
				pMain.Controls.Remove(control);
				control.Dispose();
				control = null;
			}
			
			switch (cbNdefType.SelectedIndex)
			{
				case 0:
					/* Text */
					control = new RtdTextControl();
					break;

				case 1:
					/* URI */
					control = new RtdUriControl();
					break;

				case 2:
					/* SmartPoster */
					control = new RtdSmartPosterControl();
					break;

				case 3:
					/* vCard */
					control = new RtdVCardControl();
					break;

				case 4:
					/* Arbitrary MIME Media (text) */
					control = new RtdMediaControl();
					break;
					
				case 5:
					/* WifiHandover */
					control = new RtdWifiHandoverControl();
					break;
										
				default :
					break;
			}
			
			if (control != null)
			{
				control.Dock = DockStyle.Fill;
				pMain.Controls.Add(control);
			}
		}
		
		void MiShowConsoleCheckedChanged(object sender, EventArgs e)
		{
			if (!ready)
				return;
			
			Settings s = new Settings();
			s.ShowConsole = miShowConsole.Checked;
			s.Save();
			if (miShowConsole.Checked)
			{
				SpringCardUtils.SystemConsole.Show();
				Trace.Listeners.Add(new ConsoleTraceListener());
			} else
			{
				Trace.Listeners.Clear();
				SpringCardUtils.SystemConsole.Hide();
			}
		}
		
		bool AnnounceReceiver(bool yes)
		{
			if (yes)
				Trace.WriteLine("Announce receiver: yes");
			else
				Trace.WriteLine("Announce receiver: no");
			if (!LlcpInitiator.AnnounceSnepServer(reader, yes))
			{
				MessageBox.Show("Warning: problem with ATR GI");
				return false;
			}
			return true;
		}
		
		void CbModeCheckedChanged(object sender, EventArgs e)
		{
			if (rbSendOnly.Checked)
			{
				status = Status.SendOnly_Init;
			}
			else if (rbSendThenRecv.Checked)
			{
				status = Status.SendThenRecv_Init;
			}
			else if (rbRecvOnly.Checked)
			{
				status = Status.RecvOnly_Init;
			}
			else if (rbRecvThenSend.Checked)
			{
				status = Status.RecvThenSend_Init;
			}
						
			switch (status)
			{
				case Status.SendOnly_Init :
				case Status.SendThenRecv_Init :
				case Status.RecvThenSend_Init :
					/* We could send */
					cbNdefType.Enabled = true;
					pMain.Enabled = true;
					break;
					
				case Status.RecvOnly_Init :
					/* We couldn't */
					cbNdefType.Enabled = false;
					pMain.Enabled = false;
					break;
					
				default :
					break;
			}
			
			PerformStatusChange();			
		}
	}
}
