#define _CRT_SECURE_NO_WARNINGS
#include "pcsc.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "cardware/calypso/calypso_api.h"

void get_line(char *s, DWORD n);

P_CALYPSO_CTX ctx = NULL;
char *parser_output = NULL;
#define PARSER_OUTPUT_SIZE 64*1024   /* 64 ko, must be enough ! */

void exit_handler(void)
{
  PcscCleanup();

  if (ctx != NULL)
  {
    CalypsoDestroyContext(ctx);
    ctx = NULL;
  }

  if (parser_output != NULL)
  {
    free(parser_output);
    parser_output = NULL;
  }
}

int main(int argc, char **argv)
{
  BYTE atr[32];
  DWORD atrlen;
  int i;
  DWORD j;
  DWORD reader_count, reader_index, card_index = 0;
  CALYPSO_RC rc;
  SCARDHANDLE card_handle;
  char buffer[64];

  char *xml_filename = NULL;

  atexit(exit_handler);

  printf("SpringCard PC/SC SDK\n");
  printf("\n");
  printf("Calypso demo software : XML dump of '1TIC.ICA' card application\n");
  printf("---------------------------------------------------------------\n\n");
  printf("(c) 2007-2010 SpringCard / Pro Active SAS - www.springcard.com\n\n");

  CalypsoGetVersion(buffer, sizeof(buffer));
  printf("Using DLL %s\n\n", buffer);

  for (i=1; i<argc; i++)
  {
    if (!strcmp("-o", argv[i]))
    {
      printf("found -o\n");
      if (++i<argc)
      {
        xml_filename = argv[i];
        printf("Selecting output file '%s'\n", xml_filename);
      } else
      {
        printf("Syntax error\n");
        exit(EXIT_FAILURE);
      }
    } else
    if (!strcmp("-v", argv[i]))
    {
      CalypsoSetTraceLevel(255);
      CalypsoSetTraceFile("stdout");
    }
  }

  ctx = CalypsoCreateContext();
  parser_output = malloc(PARSER_OUTPUT_SIZE);

  /* Start the PC/SC stuff */
  if (!PcscStartup())
  {
    printf("Failed to create PC/SC context");
    exit(EXIT_FAILURE);
  }

  printf("Enumerating the readers...\n");

  /* Get the list of readers */
  reader_count = PcscReaderCount();
  switch (reader_count)
  {
    case 0  : printf("No PC/SC reader, exiting\n");
              exit(EXIT_SUCCESS);
              break;
    case 1  : printf("Found 1 PC/SC reader\n");
              break;
    default : printf("Found %ld PC/SC readers\n", reader_count);
              break;
  }

  /* Display every reader */
  for (reader_index=0; reader_index<reader_count; reader_index++)
  {
    printf("[%ld] : %s\n", reader_index+1, PcscReaderName(reader_index));

    if (PcscCardPresent(reader_index))
    {
      if (reader_index == 0)
      {
        /* There's a card in the first reader, select it by default */
        card_index = reader_index+1;
      }

      /* Display the ATR (if some) */
      if (PcscCardAtr(reader_index, atr, &atrlen))
      {
        printf("\tCard present, ATR=");
        for (j=0; j<atrlen; j++)
          printf("%02X", atr[j]);
        printf("\n");
      } else
      {
        printf("\tCard present, but no ATR available\n");
      }
    } else
      printf("\tNo card in the reader\n");
  }

  /* Ask the user to select the reader where the CALYPSO card is, or will be */
  buffer[0] = '\0';
  for (;;)
  {
    if ((card_index > 0) && (card_index <= reader_count)) sprintf(buffer, "%ld", card_index);
    printf("\nSelect the PC/SC reader holding the CALYPSO card [%s] ", buffer);
    get_line(buffer, sizeof(buffer));

    if (strlen(buffer))
    {
      if ((buffer[0] == 'q') || (buffer[0] == 'Q')) exit(EXIT_FAILURE);
      card_index = atoi(buffer);
    }
    if ((card_index > 0) && (card_index <= reader_count))
      break;
  } 

  card_index--;
  
  /* Confirm there's a card in the reader, if not we'll have to wait */
  if (!PcscCardPresent(card_index))
  {
    /* Wait for a card */
    printf("Put your CALYPSO card on reader %s\n", PcscReaderName(card_index));
    printf("(Ctrl+C to cancel)\n");

    if (!PcscWaitCard(card_index, atr, &atrlen))
    {
      printf("Failed !\n");
      exit(EXIT_FAILURE);
    }

    printf("\tCard inserted, ATR=");
    for (j=0; j<atrlen; j++)
      printf("%02X", atr[j]);
    printf("\n");
  }

  /* Connect to the card */
  if (!PcscConnect(card_index, &card_handle))
  {
    printf("Failed to connect to the card.\n");
    exit(EXIT_FAILURE);
  }

  printf("Connected !\n");

  /* Try to bind our CALYPSO library to the card */
  rc = CalypsoCardBindPcsc(ctx, card_handle);
  if (rc)
  {
    printf("CalypsoCardBindPcsc failed, error %04X.\n", (unsigned int) rc);
    exit(EXIT_FAILURE);
  }

  /* Check this card is actually CALYPSO (ATR parsing or SELECT APPLICATION + FCI parsing) */
  rc = CalypsoCardActivate(ctx, NULL, 0);
  if (rc)
  {
    if (rc == CALYPSO_CARD_NOT_SUPPORTED)
    {
      printf("This is not a CALYPSO card !\n");
    } else
    {
      printf("CalypsoCardActivate failed, error %04X.\n", (unsigned int) rc);
    }
    exit(EXIT_FAILURE);
  }

  /* OK, let's explore the card */
  /* -------------------------- */
  printf("\nExploring the CALYPSO card...\n");

  /* Configure the parser output */
  CalypsoSetXmlOutputStr(ctx, parser_output, PARSER_OUTPUT_SIZE);

  /* Initialise the timer */
  CalypsoBench(TRUE);

  /* Actual exploration */
  rc = CalypsoExploreAndParse(ctx);
  if (rc)
  {
    printf("CalypsoExploreAndParse failed, error %04X.\n", (unsigned int) rc);
    exit(EXIT_FAILURE);
  }

  /* Done ! */
  printf("CALYPSO card explored in %ldms\n", CalypsoBench(TRUE));

  /* Stop the parser */
  CalypsoClearOutput(ctx);

  /* Drop card connection */
  CalypsoCardDispose(ctx);
  PcscDisconnect(card_handle, SCARD_EJECT_CARD);

  /* Now display card's data */
  /* ----------------------- */

  if (xml_filename != NULL)
  {
    FILE *fp;

    fp = fopen(xml_filename, "wt");
    if (fp == NULL)
    {
      printf("Failed to open output file '%s'\n", xml_filename);
      exit(EXIT_FAILURE);
    }

    fprintf(fp, "<calypso_explorer>\n");

    fprintf(fp, parser_output);

    fprintf(fp, "</calypso_explorer>\n");

    fprintf(fp, "\n");
    fclose(fp);

    printf("Output saved to '%s'\n", xml_filename);
  } else
  {
    printf(parser_output);
    printf("\n");
  }


  exit(EXIT_SUCCESS);
}


void get_line(char *s, DWORD n)
{
  DWORD w = 0;
  int c;

  memset(s, '\0', n);
  while ((c = fgetc (stdin)) != '\n' && c != EOF)
  {
    if (w < (n - 1))
    {
      s[w] = c;
      w++;
    }
  }
}
