/**h* NFCTools/nfc_type_4.c
 *
 * NAME
 *   nfc_type_4.c
 *
 * DESCRIPTION
 *   Support of NFC Type 4 Tag
 *
 * COPYRIGHT
 *   (c) 2011 PRO ACTIVE SAS - See LICENCE.txt for licence information
 *
 * AUTHOR
 *   Johann Dantant / PRO ACTIVE for www.springcard.com
 *
 * HISTORY
 *   JDA 17/01/2011 : first public release
 *
 **/
#include "nfc.h"
#include "nfc_card_i.h"

BOOL nfc_card_desfire_prepare(nfc_card_t card)
{
  (void) (card);

  printf("\nSorry, the 'prepare' function for Desfire cards is not implemented in this tool.\n");
	printf("Please use nfc_desfire tool instead.\n\n");
  return FALSE;
}

BOOL nfc_card_desfire_lock(nfc_card_t card)
{
  (void) (card);

  printf("\nSorry, the 'lock' function for Desfire cards is not implemented in this tool.\n");
	printf("Please use nfc_desfire tool instead.\n\n");
  return FALSE;
}

BOOL nfc_card_desfire_recognize(nfc_card_t card)
{
  unsigned char  buffer[15];

	unsigned short cc_len;
	unsigned char  mapping_version;

  if (nfc_verbose)
		printf("\tthis is a Desfire chip\n");

	card->type = CARD_NXP_DESFIRE_EV1;

  if (!nfc_card_select_nfc_application(card))
	{
		if (nfc_verbose)
			printf("\tfailed to select the NFC application\n");
    return FALSE;
	}

  if (!nfc_card_select_file(card, NFC_NDEF_7816_CC_EF_ID))
	{
		if (nfc_verbose)
			printf("\tfailed to select the CC file\n");
    return FALSE;
	}

  if (!nfc_card_read_binary(card, 0, buffer, 15))
	{
		if (nfc_verbose)
			printf("\tfailed to read the CC file\n");
    return FALSE;
	}

	cc_len = (buffer[0] << 8) | (buffer[1]);
	mapping_version = buffer[2];
	card->max_le = (buffer[3] << 8) | buffer[4];
	card->max_lc = (buffer[5] << 8) | buffer[6];

	if (((cc_len == 0x0000) && (mapping_version == 0x00) && (card->max_le == 0x0000) && (card->max_lc == 0x0000))
   || ((cc_len == 0xFFFF) && (mapping_version == 0xFF) && (card->max_le == 0xFFFF) && (card->max_lc == 0xFFFF)))
	{
		if (nfc_verbose)
			printf("\tCC file is empty\n");
    return FALSE;
  }

	if ((cc_len < 0x000F) || (cc_len > 0xFFFE))
	{
		if (nfc_verbose)
			printf("\tinvalid length in CC file\n");
    return FALSE;
	}

	if ((buffer[7] != NDEF_FILE_CONTROL_TAG) || (buffer[8] != NDEF_FILE_CONTROL_LEN))
	{
		if (nfc_verbose)
			printf("\tinvalid file control TLV in CC file\n");
    return FALSE;
  }

	if (nfc_verbose)
		printf("\tvalid CC, version=%02X, max_Le=%d, max_Lc=%d\n", mapping_version, card->max_le, card->max_lc);

	card->prepared = TRUE;

	if (buffer[13] != 0x00)
	{
		if (nfc_verbose)
			printf("\tCC says the NDEF is not readable\n");
    return FALSE;
	}

	card->file_id  = (buffer[9] << 8) | buffer[10];
	card->capacity = (buffer[9] << 11) | buffer[12];
	card->writable = (buffer[14] == 0x00) ? TRUE : FALSE;

  if (!nfc_card_select_file(card, card->file_id))
	{
		if (nfc_verbose)
			printf("\tfailed to select the NDEF file\n");
    return FALSE;
	}

  if (!nfc_card_read_binary(card, 0, buffer, 2))
	{
		if (nfc_verbose)
			printf("\tfailed to read the NDEF file\n");
    return FALSE;
	}

	if ((buffer[0] == 0x00) && (buffer[1] == 0x00))
	  card->empty = TRUE;

  return TRUE;
}


BOOL nfc_card_desfire_write_content(nfc_card_t card, unsigned char *content, size_t content_length)
{
  BYTE b[2];
	size_t i, l;

  if ((card == NULL) || (card->type != CARD_NXP_DESFIRE_EV1))
    return FALSE;

  if (!nfc_card_select_file(card, card->file_id))
	{
		if (nfc_verbose)
			printf("\tfailed to select the NDEF file\n");
    return FALSE;
	}

  for (i=0; i<content_length; )
	{
	  if (i>card->capacity)
		  break;
		l = content_length - i;
		if (l > card->max_lc)
		  l = card->max_lc;
		if (l > 254)
		  l = 254;
		if (!nfc_card_update_binary(card, (unsigned short) (i+2), &content[i], (unsigned char) l))
		{
			if (!nfc_silent)
				printf("Failed to write at address %d into the NDEF file\n", i+2);
			return FALSE;
		}

    i += l;
	}

  b[0] = (BYTE) (content_length >> 8);
	b[1] = (BYTE) (content_length);
	if (!nfc_card_update_binary(card, 0, b, 2))
	{
		if (!nfc_silent)
			printf("Failed to write at address 0 into the NDEF file\n");
		return FALSE;
	}

	return TRUE;
}


