﻿/*
 * Created by SharpDevelop.
 * User: jerome.i
 * Date: 14/06/2013
 * Time: 13:47
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
using System.Threading;
using SpringCardPCSC;
using SpringCardApplication;
using SpringCardMemoryCard;
using Be.Windows.Forms;

namespace MemoryCardTool
{
	public partial class MainForm : Form
	{
		ShadowForm shade;
		SCardReader reader = null;
		SCardChannel cardchannel = null;
		
		MemoryCard card = null;
		Thread cardthread;
		
		private Settings settings = null;
		
		public MainForm()
		{
			InitializeComponent();
			settings = new Settings();
		}
		
		
		public void setReader(string readerName)
		{
			
			eReaderName.Text = readerName;
			eReaderStatus.Text = "";
			eCardAtr.Text = "";
			
			
			reader = new SCardReader(readerName);
			reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChanged));
			
		}

		delegate void ReaderStatusChangedInvoker(uint ReaderState, CardBuffer CardAtr);
		void ReaderStatusChanged(uint ReaderState, CardBuffer CardAtr)
		{
			/* The ReaderStatusChanged function is called as a delegate (callback) by the SCardReader object    */
			/* within its backgroung thread. Therefore we must use the BeginInvoke syntax to switch back from   */
			/* the context of the background thread to the context of the application's main thread. Overwise   */
			/* we'll get a security violation when trying to access the window's visual components (that belong */
			/* to the application's main thread and can't be safely manipulated by background threads).         */
			if (InvokeRequired)
			{
				this.BeginInvoke(new ReaderStatusChangedInvoker(ReaderStatusChanged), ReaderState, CardAtr);
				return;
			}
			
			eReaderStatus.Text = SCARD.ReaderStatusToString(ReaderState);
			
			if (CardAtr != null)
			{
				eCardAtr.Text = CardAtr.AsString(" ");
			} else
			{
				eCardAtr.Text = "";
			}
			
			if (ReaderState == SCARD.STATE_UNAWARE)
			{
				if (cardchannel != null)
				{
					cardchannel.Disconnect();
					cardchannel = null;
				}
				
				MessageBox.Show("The reader we were working on has disappeared from the system. This application will terminate now.",
				                "The reader has been removed",
				                MessageBoxButtons.OK,
				                MessageBoxIcon.Exclamation);
				
				Application.Exit();
				
			} else
				if ((ReaderState & SCARD.STATE_EMPTY) != 0)
			{
				add_log_text("Reader: EMPTY");
				
				if (cardchannel != null)
				{
					cardchannel.Disconnect();
					cardchannel = null;
				}
				
			} else
				if ((ReaderState & SCARD.STATE_UNAVAILABLE) != 0)
			{
				add_log_text("Reader: UNAVAILABLE");
			} else
				if ((ReaderState & SCARD.STATE_MUTE) != 0)
			{
				add_log_text("Reader: MUTE");
			} else
				if ((ReaderState & SCARD.STATE_INUSE) != 0)
			{
				add_log_text("Reader: INUSE");
			} else
				if ((ReaderState & SCARD.STATE_PRESENT) != 0)
			{
				add_log_text("Reader: PRESENT");
				
				if (cardchannel == null)
				{
					/* New card -> leave edit mode */
					//setEditable(false);
					
					cardchannel = new SCardChannel(reader);
					
					if (cardchannel.Connect())
					{
						add_log_text("Connected to the card");
						cardthread = new Thread(card_read_proc);
						cardthread.Start();
						
					} else
					{
						add_log_text("Connection to the card failed");
						
						shade = new ShadowForm();
						shade.ShowShadow(this);

						MessageBox.Show("Failed to connect to the card in the reader : please check that you don't have another application running in background that tries to work with the smartcards in the same time");
						if (shade != null)
						{
							shade.Dispose();
							shade = null;
						}
						


						cardchannel = null;
					}
				}
				
			}
			
		}
		
		private void card_read_proc()
		{
			
			MemoryCard card = null;
			string msg = null;
			this.BeginInvoke(new OnEraseScreenInvoker(EraseScreen));
			
			add_log_text("Is the card a memory card ?");
			
			/*
			 * 1st step, is this a memory card ?
			 * ---------------------------------
			 */
			
			if ( ! MemoryCard.Recognize(this, cardchannel, out card, out msg))
				add_log_text("This card is not a memory card");

			
			/*
			 * 2nd step, tell the application we've got something, and die
			 * -----------------------------------------------------------
			 */
			
			if (card != null)
			{
				this.BeginInvoke(new OnCardReadInvoker(onCardRead), card);

			} else
			{
				this.BeginInvoke(new OnErrorInvoker(OnError), msg, "This is not a memory card");
			}
			
		}
		
		delegate void OnErrorInvoker(string text, string caption);
		void OnError(string text, string caption)
		{
			add_log_text("Error: " + text);
			if (shade == null)
			{
				shade = new ShadowForm();
				shade.ShowShadow(this);
			}
			
			EraseScreen();

			
			MessageBox.Show(text, caption);
			
			if (shade != null)
			{
				shade.Dispose();
				shade = null;
			}
		}
		
		delegate void OnEraseScreenInvoker();
		void EraseScreen()
		{
			pMain.Controls.Clear();
			lbCardType.Text = "";
			lbSpecify.Text = "";
			lbSpecifyP1.Text = "";
			lbSpecifyP2.Text = "";
			lbNbSectors.Text = "";
			eP1.Visible = false;
			eP2.Visible = false;
			
			mkTxtNbSectors.Visible = false;
			btUpdate.Visible = false;
		}
		
		delegate void OnEraseMainPannelInvoker();
		void EraseMainPannel()
		{
			pMain.Controls.Clear();
		}
		
		
		delegate void OnCardReadInvoker(MemoryCard _card);
		void onCardRead(MemoryCard _card)
		{
			card = _card;
			
			add_log_text("Read terminated");
			
			if (card == null)
			{
				MessageBox.Show("Internal error, card is null!");
				return;
			}
			
			if (card is MemoryCardWithoutSectors)
			{
				printMemoryCardWithoutSectors();
			} else
				if (card is MemoryCardMifareClassic)
			{
				printMemoryCardWithSectors();
			}
			

		}
		
		void printMemoryCardWithoutSectors()
		{
			if (card is MemoryCardWithoutSectors)
			{
				MemoryCardWithoutSectorsControl control = new MemoryCardWithoutSectorsControl((MemoryCardWithoutSectors) card);
				pMain.Controls.Add(control);
				lbCardType.Text = "Recognized card: " + card.CardType;
				lbSpecify.Visible = true;
				lbSpecify.Text = "Specify maximum values: ";
				lbSpecifyP1.Text = "P1";
				eP1.Visible = true;
				eP1.Enabled = true;
				lbSpecifyP2.Text = "P2";
				eP2.Visible = true;
				eP2.Enabled = true;
				lbNbSectors.Text = "";
				/*
				eP1.Value = (byte) (((MemoryCardWithoutSectors) card).Pages.Count >> 8);
				eP2.Value = (byte) (((MemoryCardWithoutSectors) card).Pages.Count & 0xFF);
				
				if ( (card.CardType.Equals("PIX_NN_INNOVISION_JEWEL")) || (card.CardType.Equals("PIX_NN_INNOVISION_TOPAZ")))
				{
					MessageBox.Show("ici");
					int lastlineP1P2 = ((MemoryCardWithoutSectors) card).P1P2.Count;
					eP1.Value = (byte) ((MemoryCardWithoutSectors) card).P1P2[lastlineP1P2-1][0];
					eP2.Value = (byte) ((MemoryCardWithoutSectors) card).P1P2[lastlineP1P2-1][1] + ((MemoryCardWithoutSectors) card).Pages[lastlineP1P2-1].Length;
				} else
				{
					int lastlineP1P2 = ((MemoryCardWithoutSectors) card).P1P2.Count;
					eP1.Value = (byte) ((MemoryCardWithoutSectors) card).P1P2[lastlineP1P2-1][0];
					eP2.Value = (byte) ((MemoryCardWithoutSectors) card).P1P2[lastlineP1P2-1][1];
				}
				 */
				eP1.Value = ((MemoryCardWithoutSectors) card).MaxP1;
				eP2.Value = ((MemoryCardWithoutSectors) card).MaxP2;

				
				mkTxtNbSectors.Visible = false;
				btUpdate.Visible = true;
				
				lbSpecify.Location = new Point(60, 33);
				lbSpecifyP1.Location = new Point(217, 33);
				eP1.Location = new Point(253, 35);
				lbSpecifyP2.Location = new Point(333, 33);
				eP2.Location = new Point(382, 35);
				btUpdate.Location = new Point(470, 32);
				lbNbSectors.Location = new Point(514, 33);
				mkTxtNbSectors.Location = new Point(641,35);
				
			}
		}
		
		void printMemoryCardWithSectors()
		{
			if (card is MemoryCardMifareClassic)
			{
				MemoryCardWithSectorsControl control = new MemoryCardWithSectorsControl(settings, (MemoryCardMifareClassic) card);
				pMain.Controls.Add(control);
				lbCardType.Text = "Recognized card: " + card.CardType;
				lbSpecify.Text = "";
				lbSpecify.Visible = false;
				lbSpecifyP1.Text = "P1";
				eP1.Visible = true;
				eP1.Enabled = false;
				lbSpecifyP2.Text = "P2";
				eP2.Visible = true;
				eP2.Enabled = false;
				lbNbSectors.Text = "Number of sectors:";
				mkTxtNbSectors.Text = ((MemoryCardMifareClassic) card).NbSectors.ToString();
				mkTxtNbSectors.Visible = true;
				btUpdate.Visible = true;
				UInt32 address;
				if (((MemoryCardMifareClassic) card).NbSectors <= 32)
				{
					address = (UInt32) (((MemoryCardMifareClassic) card).NbSectors * 4 - 1);
				} else
				{
					address = (UInt32) (32*4 + (((MemoryCardMifareClassic) card).NbSectors - 32)*	16 - 1);
				}
				
				eP1.Value = (byte) address >> 8;
				eP2.Value = (byte) address & 0x00FF;
				
				lbNbSectors.Location = new Point(60, 33);
				mkTxtNbSectors.Location = new Point(187, 35);
				btUpdate.Location = new Point(217, 32);
				
				int offset = 37;
				lbSpecifyP1.Location = new Point(217 + offset, 33);
				eP1.Location = new Point(253 + offset, 35);
				lbSpecifyP2.Location = new Point(333 + offset, 33);
				eP2.Location = new Point(382 + offset, 35);
				
			}
		}
		
		void MainFormShown(object sender, EventArgs e)
		{
			SplashForm splash = new SplashForm();
			splash.ShowDialog();
			
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			
			if (readerSelect.SelectedReader == null)
			{
				readerSelect.Preselect("SpringCard|contactless");
				for (;;)
				{
					readerSelect.ShowDialog();
					if (readerSelect.SelectedReader != null)
						break;
					
					if (MessageBox.Show("This application can't run without a reader. Do you want to leave the application now ?",
					                    "No reader selected",
					                    MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
					{
						Application.Exit();
						return;
					}
				}
			}
			
			shade.Dispose();
			shade = null;
			
			setReader(readerSelect.SelectedReader);
		}
		
		void MainFormLoad(object sender, EventArgs e)
		{
			shade = new ShadowForm();
			shade.ShowShadow(this);
		}
		
		#region Log management
		private string log_txt = "";
		public bool IsLogFormOn = false;
		public LogForm log_box;
		
		void ViewLogToolStripMenuItemClick(object sender, EventArgs e)
		{
			ShowLogForm();
		}
		
		void ShowLogForm()
		{
			if (!IsLogFormOn)
			{
				if (shade == null)
				{
					shade = new ShadowForm();
					shade.ShowShadow(this);
				}
				
				LogForm log = new LogForm();
				log.MainForm = this;
				log.specify_log_text(log_txt);
				
				log.ShowDialog();
				shade.Dispose();
				shade = null;
				
			}
			else
			{
				log_box.specify_log_text(log_txt);
				log_box.Focus();
			}
		}
		
		public void erase_log_txt()
		{
			log_txt="";
		}
		
		public void add_log_text(string s)
		{
			log_txt += DateTime.Now + ": " + s + "\r\n";

		}
		#endregion
		
		
		void ChangeReaderToolStripMenuItemClick(object sender, EventArgs e)
		{
			shade = new ShadowForm();
			shade.ShowShadow(this);
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			readerSelect.SelectedReader = eReaderName.Text;
			readerSelect.ShowDialog();
			
			shade.Dispose();
			shade = null;

			if (readerSelect.SelectedReader != null)
				setReader(readerSelect.SelectedReader);
		}
		
		void ExitToolStripMenuItemClick(object sender, EventArgs e)
		{
			Close();
		}
		

		
		void AboutToolStripMenuItemClick(object sender, EventArgs e)
		{
			shade = new ShadowForm();
			shade.ShowShadow(this);
			AboutForm about = new AboutForm();
			about.ShowDialog();
			shade.Dispose();
			shade = null;
		}
		
		void BtUpdateClick(object sender, EventArgs e)
		{
			
			if (card is MemoryCardMifareClassic)
			{
				int nb = int.Parse(mkTxtNbSectors.Text);
				((MemoryCardMifareClassic) card).setMaxSectors(nb);
				cardthread = new Thread(read_card_again);
				cardthread.Start();
			} else
				if (card is MemoryCardWithoutSectors)
			{
				if (! ((MemoryCardWithoutSectors) card).SetMaxAddress((byte)(eP1.Value), (byte)(eP2.Value)))
				{
					MessageBox.Show("Error setting maximum P1 and P2 parameters", "Error in parameters", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				
				cardthread = new Thread(read_card_again);
				cardthread.Start();
			}
			
		}
		
		
		private void read_card_again()
		{
			this.BeginInvoke(new OnEraseMainPannelInvoker(EraseMainPannel));
			
			add_log_text("Read card again");
			
			/*
			 * 1st step, is this a memory card ?
			 * ---------------------------------
			 */
			
			if ( ! card.ReadAgain())
			{
				add_log_text("This card cannot be read again");
				return;
			}

			/*
			 * 2nd step, tell the application we've got something, and die
			 * -----------------------------------------------------------
			 */
			
			if (card != null)
			{
				this.BeginInvoke(new OnCardReadInvoker(onCardRead), card);

			} else
			{
				this.BeginInvoke(new OnErrorInvoker(OnError), "Error reading the card", "This is not a memory card");
			}

		}
		
		
		void MainFormFormClosed(object sender, FormClosedEventArgs e)
		{
			if (settings != null)
				settings.Save();
		}
	}
	
}
