﻿/*
 * Created by SharpDevelop.
 * User: jerome.i
 * Date: 20/06/2013
 * Time: 14:36
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Generic;
using SpringCardMemoryCard;
using SpringCardApplication;
using Be.Windows.Forms;
	
namespace MemoryCardTool
{
	/// <summary>
	/// Description of SectorControl.
	/// </summary>
	public partial class SectorControl : UserControl
	{
		private MifareSector sector;
		private ToolTip toolTip;
		Settings settings;
		
		public SectorControl(Settings _settings, MifareSector _sector)
		{
			InitializeComponent();
			settings = _settings;
			sector = _sector;
			PrintData();
			
		}

		void PrintData()
		{
			HexBoxContent._mifare_sector = true;
			lbSector.Text = sector.Sector.ToString();
			btWriteToCard.Enabled = false;
			btTryDifferentKeys.Enabled = false;
			btTryDifferentKeys.Visible = false;
			HexBoxContent.Enabled = true;
			btWriteToCard.Visible = true;
			btChangeAccesConditions.Visible = true;
			
			toolTip = new ToolTip();
			toolTip.SetToolTip(btChangeAccesConditions, "Change keys and access conditions");
			toolTip.SetToolTip(btTryDifferentKeys, "Unlock sector with specific keys");
			toolTip.SetToolTip(btWriteToCard, "Write current modifications to the card");
			
			if (sector.Blocks != null)
			{
				byte[] content_bytes = new byte[sector.Blocks[0].Length * sector.Blocks.Count];
				if (sector.Blocks.Count > 10)
				{
					HexBoxContent.Size = new Size(HexBoxContent.Size.Width, 228);
				}
				
				for (int i = 0; i <sector.Blocks.Count; i++)
					Array.ConstrainedCopy(sector.Blocks[i], 0, content_bytes, i*sector.Blocks[0].Length, sector.Blocks[0].Length);
				
				DynamicByteProvider p;
		    p = new DynamicByteProvider(content_bytes);
		  	HexBoxContent.ByteProvider = p;
		  			  			
			} else
			{
				HexBoxContent.Enabled = false;
				btWriteToCard.Visible = false;
				btTryDifferentKeys.Enabled = true;
				btTryDifferentKeys.Visible = true;
				btChangeAccesConditions.Visible = false;
			}			
		}
		
		void HexBoxContentKeyDown(object sender, KeyEventArgs e)
		{
			btWriteToCard.Enabled = true;
			if (sector.Sector == 0)
				toolTip.SetToolTip(btWriteToCard, "Write current modifications to the card - block 0 can't be changed");
		}
		
		void BtWriteToCardClick(object sender, EventArgs e)
		{
			byte[] b;
			List<byte[]> new_blocks;
			byte[] tmp;
			DynamicByteProvider p;
			
			/* Get the bytes	*/
			p = (DynamicByteProvider) HexBoxContent.ByteProvider;
      b = new byte[p.Length];
      
      for (int i=0; i<p.Length; i++)
        b[i] = p.ReadByte(i);
			
      new_blocks = new List<byte[]>();
      
			/* Sector 0 : don't change first block	*/
			if (sector.Sector == 0)
			{
				new_blocks.Add(sector.Blocks[0]);
				for (int i=16; i<b.Length ; i+=16)
	      {
					if ((i+16) <= b.Length)
					{
						tmp = new byte[16];
						Array.ConstrainedCopy(b, i, tmp, 0, 16);
	      		new_blocks.Add(tmp);	
					} else
					{
						MessageBox.Show("The number of bytes in the table doesn't correspond with the capacity of the card.", "Error: wrong number of bytes", MessageBoxButtons.OK, MessageBoxIcon.Error);
						btWriteToCard.Enabled = false;
    				return;
					}

	      }		
			} else
			{
				for (int i=0; i<b.Length ; i+=16)
	      {	
	      	if ((i+16) <= b.Length)
	      	{
		      	tmp = new byte[16];
		      	Array.ConstrainedCopy(b, i, tmp, 0, 16);
		      	new_blocks.Add(tmp);		      		
	      	} else
	      	{
	      		MessageBox.Show("The number of bytes in the table doesn't correspond with the capacity of the card.", "Error: wrong number of bytes", MessageBoxButtons.OK, MessageBoxIcon.Error);
    				btWriteToCard.Enabled = false;
    				return;
	      	}
	      }				
			}
			
			/*
			string s = "";
			for (int i=0; i<new_blocks.Count; i++)
			{
				for (int j=0; j<new_blocks[i].Length; j++)
					s += String.Format("{0:x02}", new_blocks[i][j]);
				
				s+="\n";
			}
			MessageBox.Show("new blocks :\n" + s);
			*/
			
			/* Checks	*/
			if (new_blocks.Count != sector.Blocks.Count)
			{
				MessageBox.Show("The number of bytes in the table doesn't correspond with the capacity of the sector.", "Error: wrong number of bytes", MessageBoxButtons.OK, MessageBoxIcon.Error);
				btWriteToCard.Enabled = false;
				return;
			}
			
			
			sector.SetNewBlocks(new_blocks);
			if (!sector.WriteNewBlocks())
			{
				MessageBox.Show("The writing has failed", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				
				/* Give the chance to specify other keys	*/
				btTryDifferentKeys.Visible = true;
				btTryDifferentKeys.Enabled = true;
				btWriteToCard.Visible = false;
				btWriteToCard.Enabled = false;
				btChangeAccesConditions.Visible = false;
				HexBoxContent.Enabled = false;
				
     	} else
     	{
     		MessageBox.Show("The sector has been successfully updated !", "Writing OK", MessageBoxButtons.OK);
     	}
     
     	btWriteToCard.Enabled = false;
						
		}
		
		void BtChangeAccesConditionsClick(object sender, EventArgs e)
		{
			ChangeSectorTrailer change = new ChangeSectorTrailer(settings, sector);
			DialogResult r = change.ShowDialog();
			if (r == DialogResult.Cancel)
				return;
			
			if (sector.WriteSectorTrailer())
			{
				MessageBox.Show("The access conditions have been updated !", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
			} else
			{
				MessageBox.Show("The modification of access conditions has failed.\nYou may need to specify a writing key.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				
				/* Give the chance to specify other keys	*/
				btTryDifferentKeys.Visible = true;
				btTryDifferentKeys.Enabled = true;
				btWriteToCard.Visible = false;
				btWriteToCard.Enabled = false;
				btChangeAccesConditions.Visible = false;
				HexBoxContent.Enabled = false;
				
			}
		}
		
		void btTryDifferentKeysClick(object sender, EventArgs e)
		{
			UnlockForm unlock = new UnlockForm(settings, this);
			DialogResult r = unlock.ShowDialog();
			if (r == DialogResult.Cancel)
				return;
			
			if (sector.ReadWithGivenKeys())
			{
				PrintData();
			} else
			{
				MessageBox.Show("This sector can't be read with the specified key(s).", "Incorrect key(s)", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			
			
		}
	
		public void setKeyA(byte[] KeyA)
		{
			sector.setKeyA(KeyA);
		}
		
		public void setKeyB(byte[] keyB)
		{
			sector.setKeyB(keyB);
		}
		
	}
}
