using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

using SpringCardPCSC;
using SpringCardApplication;

namespace csScriptor
{
  public partial class MainForm : Form
  {
		
		ShadowForm shade;
		SCardReader reader = null;
		SCardChannel cardchannel = null;  	
		
		public string ScriptFileName = "";
		private bool ASCIIFormat = true;
		private bool HEXFormat = true;

 
    public MainForm()
    {
      InitializeComponent();         
    }

    void MainFormLoad(object sender, EventArgs e)
    {
    	shade = new ShadowForm();
			shade.ShowShadow(this);
    }
    

    void MainFormShown(object sender, EventArgs e)
		{
			SplashForm splash = new SplashForm();
			splash.ShowDialog();
			
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			
			if (readerSelect.SelectedReader == null)
			{
			  readerSelect.Preselect("SpringCard|contactless");
			  for (;;)
			  {
			    readerSelect.ShowDialog();
			    if (readerSelect.SelectedReader != null)
			      break;
			    
			    if (MessageBox.Show("This application can't run without a reader. Do you want to leave the application now ?",
			                        "No reader selected",
			                        MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
			    {
			      Application.Exit();
			      return;
			    }			    
			  }			  
			}
			
			shade.Dispose();
			shade = null;      
			
			setReader(readerSelect.SelectedReader);				
		}
		
		public void setReader(string readerName)
		{		  
		 	eReaderName.Text = readerName;
		  eReaderStatus.Text = "";
		  eCardAtr.Text = "";
		  
		  reader = new SCardReader(readerName);
			reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChanged));			
		}

		
		delegate void ReaderStatusChangedInvoker(uint ReaderState, CardBuffer CardAtr);		
		void ReaderStatusChanged(uint ReaderState, CardBuffer CardAtr)
		{
		  /* The ReaderStatusChanged function is called as a delegate (callback) by the SCardReader object    */
		  /* within its backgroung thread. Therefore we must use the BeginInvoke syntax to switch back from   */
		  /* the context of the background thread to the context of the application's main thread. Overwise   */
		  /* we'll get a security violation when trying to access the window's visual components (that belong */
		  /* to the application's main thread and can't be safely manipulated by background threads).         */
			if (InvokeRequired)
			{
				this.BeginInvoke(new ReaderStatusChangedInvoker(ReaderStatusChanged), ReaderState, CardAtr);
				return;				
			}			
		
			eReaderStatus.Text = SCARD.ReaderStatusToString(ReaderState);
			
			if (CardAtr != null)
			{
			  eCardAtr.Text = CardAtr.AsString(" ");
			  buttonRun.Enabled = true;
			  runSMenuItem.Enabled = true;
			  
			} else
			{
			  eCardAtr.Text = "";
			  buttonRun.Enabled = false;
			  runSMenuItem.Enabled = false;
			}
			
			if (ReaderState == SCARD.STATE_UNAWARE)
			{
			  if (cardchannel != null)
			  {
			    cardchannel.Disconnect();
			    cardchannel = null;
			  }
			  
			  MessageBox.Show("The reader we were working on has disappeared from the system. This application will terminate now.",
			                  "The reader has been removed",
			                  MessageBoxButtons.OK,
			                  MessageBoxIcon.Exclamation);
			  
			  Application.Exit();
			  
			} else
  	  if ((ReaderState & SCARD.STATE_EMPTY) != 0)
			{
			  
			  if (cardchannel != null)
			  {
			    cardchannel.Disconnect();
			    cardchannel = null;
			  }
			  
			} else
			if ((ReaderState & SCARD.STATE_UNAVAILABLE) != 0)
			{
      
			} else			  
			if ((ReaderState & SCARD.STATE_MUTE) != 0)
			{
        
			} else
	    if ((ReaderState & SCARD.STATE_INUSE) != 0)
			{
        
			} else
	    if ((ReaderState & SCARD.STATE_PRESENT) != 0)
			{ 
			  if (cardchannel == null)
			  {
  			  /* New card -> leave edit mode */    
			    cardchannel = new SCardChannel(reader);
			    
			    if (! cardchannel.Connect())
			    {
          	shade = new ShadowForm();
      		  shade.ShowShadow(this);

			      MessageBox.Show("Failed to connect to the card in the reader : please check that you don't have another application running in background that tries to work with the smartcards in the same time");
			      if (shade != null)
			      {
	            shade.Dispose();
      	    	shade = null;		      	
			      }
			      cardchannel = null;
			    
			    }
			    
			  }
			  
			}
			
		}
		
    private void ClearResult(object sender, EventArgs e)
    {
      richTextBoxResult.Clear();
    }
		
    private void RunScript(object sender, EventArgs e)
    {
      Run();
    }

    private void runSMenuItem_Click(object sender, EventArgs e)
    {
      Run();
    }

    
    public void Run()
    {

      if (cardchannel == null)
      {
      	MessageBox.Show("Please remove the card and insert it again in the reader !", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      	return;
      }
      {      	
      	/* Getting user entry	*/
        string Script = richTextBoxScript.Text;

        char[] delimiter = new char[1];
        delimiter[0] = '\n';
       
        /* seperating apdus (delimiter = '\n')	*/
        string[] lines = Script.Split(delimiter);
        richTextBoxResult.SelectionColor = Color.Black;
        richTextBoxResult.AppendText("--------------------------------------------------\n");
        richTextBoxResult.AppendText("Beginning Script Execution...\n");
        richTextBoxResult.AppendText("-----------------------------");
        string cmd = "";
        int nbLine = 0;
        foreach (string line in lines)
        {
          nbLine ++;
          
        	/* skip comment	*/
          if (line.StartsWith("#"))
            continue;

          /* skip blank lines	*/
          string apdu = line.Replace(" ","");
          if (apdu.Equals(""))
            continue;
					
          /* if line ends with \, concatenation with next line	*/
          if (apdu.EndsWith("\\"))
          {
            cmd = cmd + apdu.Remove(apdu.Length - 1);
            continue;
          }

          cmd = cmd + apdu;
					CAPDU capdu;
          try
          {
          	capdu = new CAPDU(cmd);
          } catch
          {
          	richTextBoxResult.SelectionColor = Color.Red;
          	richTextBoxResult.AppendText("\nError in APDU: '" + cmd.ToUpper() + "'\n");
          	if (cbStopOnError.Checked)
          		return;

          	cmd = "";
          	continue;
          }
					          
          richTextBoxResult.SelectionColor = Color.Black;
          richTextBoxResult.AppendText("\nSending : ");
          
          richTextBoxResult.SelectionColor = Color.Blue;
          richTextBoxResult.AppendText(capdu.AsString(" "));
          
          if (cardchannel == null	)
          {
		      	MessageBox.Show("Communication error with the card !", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
		      	return;
          }
          
          RAPDU rapdu =  cardchannel.Transmit(capdu);
          
          if (rapdu == null)
          {
		      	MessageBox.Show("Communication error with the card !", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
		      	return;
          }
          
          cmd = "";

          Color resultColor;
          bool errorOccured = false;
          if ((rapdu.SW == 0x9000) || (rapdu.SW == 0x9100) || (rapdu.SW == 0x91AF))
          {
          	resultColor = Color.Green;
          } else
          if (rapdu.SW1 != 0x6F)
          {
          	resultColor = Color.DarkOrange;
          	errorOccured = true;
          } else
          {
          	resultColor = Color.Red;
          	errorOccured = true;
          }
          
          if (HEXFormat)
          {
          	richTextBoxResult.SelectionColor = Color.Black;
          	richTextBoxResult.AppendText("\nReceived HEX: ");
          	richTextBoxResult.SelectionColor = resultColor;
          	richTextBoxResult.AppendText(rapdu.AsString(" "));
          }
          
          if (ASCIIFormat)
          {
          	richTextBoxResult.SelectionColor = Color.Black;
          	richTextBoxResult.AppendText("\nReceived ASCII: ");
          	richTextBoxResult.SelectionColor = resultColor;
          	          	
          	if (rapdu.data == null)
          	{
			      	MessageBox.Show("Communication error with the card !", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			      	return;
          	}
          	
          	
          	if (rapdu.data.GetBytes() == null)
          	{
			      	MessageBox.Show("Communication error with the card !", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			      	return;
          	}
          	
          	if (rapdu.SWString == null)
          	{
			      	MessageBox.Show("Communication error with the card !", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			      	return;
          	}
          	
          	richTextBoxResult.AppendText(ConvertToString(rapdu.data.GetBytes()) + " - " + rapdu.SWString);
          
          }
          
         	richTextBoxResult.AppendText("\n");
        
         	if (errorOccured && cbStopOnError.Checked)
          		return;
        }
        richTextBoxResult.AppendText("\n\n");
      }
      
    }

    private bool isASCIIChar(byte b)
    {
    	if ((b < 0x20) || (b>= 0x7F))
    		return false;
    	
    	return true;
    }
    		
    private string getASCIIChar(byte b)
    {
    	byte[] ab = new byte[1];
    	ab[0] = b;
    	return System.Text.Encoding.ASCII.GetString(ab);
    }
    
    private string ConvertToString(byte[] array)
    {
    	string s = "";
    	for (int i=0; i<array.Length; i++)
    	{
    		if (isASCIIChar(array[i]))
    		{
    			s += getASCIIChar(array[i]);
    		} else
    		{
    			s += " ";
    		}
    	}
    	return s;
    }
    
    private void openMenuItem_Click(object sender, EventArgs e)
    {
      openFileDialog.Filter = "Script files(*.txt)|*.txt|All files (*.*)|*.*";
      openFileDialog.FilterIndex = 1;
      openFileDialog.RestoreDirectory = true;
      openFileDialog.DefaultExt=".txt";
    	openFileDialog.ShowDialog();
    }

    private void ReadScriptFile(object sender, CancelEventArgs e)
    {
      Stream userFileStream = new FileStream(openFileDialog.FileName, 
        FileMode.Open, FileAccess.Read);
      StreamReader userFileStreamReader = new StreamReader(userFileStream);
      string userFileString = userFileStreamReader.ReadToEnd();
      richTextBoxScript.Text = userFileString;
      ScriptFileName = openFileDialog.FileName;
      
      userFileStreamReader.Close();
      userFileStream.Close();
      
    }

    private void saveFileDialog_FileOk(object sender, CancelEventArgs e)
    {
      ScriptFileName = saveFileDialog.FileName;
      writeFile();
    }

    private void saveAsMenuItem_Click(object sender, EventArgs e)
    {
      string Script = richTextBoxScript.Text;
      
      /* Is there something to save?	*/
      if (Script != "")
      {
      	saveFileDialog.Filter = "Script files(*.txt)|*.txt|All files (*.*)|*.*";
	      saveFileDialog.FilterIndex = 1;
	      saveFileDialog.RestoreDirectory = true;
	      saveFileDialog.DefaultExt=".txt";
      	saveFileDialog.ShowDialog();
      }

    }

    private void saveMenuItem_Click(object sender, EventArgs e)
    {
      string Script = richTextBoxScript.Text; 
     
      /* Is there something to save?	*/
      if (Script != "")
      {
        if (ScriptFileName == "")
        {
          saveFileDialog.Filter = "Script files(*.txt)|*.txt|All files (*.*)|*.*";
		      saveFileDialog.FilterIndex = 1;
		      saveFileDialog.RestoreDirectory = true;
		      saveFileDialog.DefaultExt=".txt";
        	saveFileDialog.ShowDialog();
        }else 
        {
          writeFile();
        }
      }
    }

    public void writeFile()
    {
      /* Getting user entry	*/
      string Script = richTextBoxScript.Text;
      char[] delimiter = new char[1];
      delimiter[0] = '\n';
      
      /* seperating apdus (delimiter = '\n')	*/
      string[] lines = Script.Split(delimiter);

      Stream writeFileStream = new FileStream(ScriptFileName, FileMode.OpenOrCreate, FileAccess.Write);
      StreamWriter writeFileStreamWriter = new StreamWriter(writeFileStream);
      
      foreach (string line in lines) 
        writeFileStreamWriter.WriteLine(line);
          
      writeFileStreamWriter.Close();
      writeFileStream.Close();
      
    }

    private void NewScriptFile(object sender, EventArgs e)
    {
      /* Erasing default file name	*/
      ScriptFileName = "";

      /* Erasing script area	*/
      richTextBoxScript.Text = "";
    }

    private void checkBoxWrap_CheckedChanged(object sender, EventArgs e)
    {
      richTextBoxScript.WordWrap = checkBoxWrap.Checked;
    }



    private void quitMenuItem_Click(object sender, EventArgs e)
    {
      Close();
    }
    private bool AskConfirmQuitAppli()
    {
      if (MessageBox.Show("Are you sure you want to quit?",
      "Confirmation",
      MessageBoxButtons.OKCancel) == DialogResult.OK)
        return true;
      return false;
    }

    private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
    {
      if (AskConfirmQuitAppli() == false)
        e.Cancel = true;
    }

    
    private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
    {

    }
    
    private void aboutCScriptorToolStripMenuItem_Click(object sender, EventArgs e)
    {
			
    	shade = new ShadowForm();
		  shade.ShowShadow(this);
	  	AboutForm about = new AboutForm();
    	about.ShowDialog();
   		shade.Dispose();
	    shade = null;
	    
    }

    private void MainForm_KeyDown(object sender, KeyEventArgs e)
    {
      if (e.KeyCode == Keys.F5)
        Run();
    }

    private void butClearRes_Click(object sender, EventArgs e)
    {
      richTextBoxResult.Clear();
    }

		
    private void richTextBoxResult_TextChanged(object sender, EventArgs e)
    {
      /* Put the cursor at the very end of the text	*/
      richTextBoxResult.Select((richTextBoxResult.Text.Length), 0);

      /* Scroll	*/
      richTextBoxResult.ScrollToCaret();

    }

    private void richTextBoxScript_TextChanged(object sender, EventArgs e)
    {
      /* Scroll	*/
      richTextBoxScript.ScrollToCaret();
    }
		
    
    void SetReader(object sender, EventArgs e)
    {
    	shade = new ShadowForm();
		  shade.ShowShadow(this);
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			readerSelect.SelectedReader = eReaderName.Text;
      readerSelect.ShowDialog();
      
   		shade.Dispose();
	    shade = null;

	    if (readerSelect.SelectedReader != null)
	      setReader(readerSelect.SelectedReader);		
    }
    

    
    void radioButton_CheckedChanged(object sender, EventArgs e)
    {
    	if (radioButtonAscii.Checked)
	    {
	    	ASCIIFormat = true;	
	    	HEXFormat = false;
	    } else
    	if (radioButtonHex.Checked)
    	{
    		ASCIIFormat = false;
    		HEXFormat = true;
    	} else
    	if (radioButtonBoth.Checked)
    	{
    		ASCIIFormat = true;
    		HEXFormat = true;    		
    	}

    }
  }
}
